var celery = require('celeri'),
service = require('./service'),
_ = require('underscore');



function unableToFetch()
{
    return console.log('Unable to fetch'.red);
}


function simpleLoad(uri, label)
{
    var ops = {};

    if(typeof uri == 'string')
    {
        ops.path = uri;
    }
    else
    {
        ops = uri;
    }

    var loading = celery.loading(label ? label + ': ' : null);
    
    service.request(ops, function(err, result) 
    { 
        loading.done(null, result && !result.errors);
    });
}



celery.on({

     /**
      * list apps 
      */
     
     'list': function()
     {
        var loading = celery.loading();
        
        service.request('/garden/apps', function(err, response)
        {
            loading.done();
            
            if(err || response.errors) return unableToFetch();
            
            var table = [];
            
            celery.drawTable(response.result, {
                columns: {
                    name: 10,
                    state: {
                        get: function(app)
                        {
                            if(!app.nodes.length) return "".yellow;
                            
                            return _.map(app.nodes, function(node)
                            {
                                //TODO: node name (hostname)
                                return node.statename.green;
                            }).join('\n');
                        }
                    },
                    /*uptime: {
                        get: function(app)
                        {
                            if(!app.nodes.length) return "".yellow;
                            
                            return _.map(app.nodes, function(node)
                            {
                                //TODO: node name (hostname)
                                return node.description.green;
                            }).join('\n');
                        }
                    },*/
                    description: {
                        get: function(app)
                        {
                            if(!app.nodes.length) return "stopped".yellow;
                            
                            return _.map(app.nodes, function(node)
                            {
                                //TODO: node name (hostname)
                                return node.description.green;
                            }).join('\n');
                        },
                        align:'left'
                    }
                    
                }
            });
            /*response.result.forEach(function(app)
            {
                
                if(!app.nodes.length)
                {
                    console.log(app.name.blue +' - not running'.red);
                }
                else
                {
                
                    for(var i = app.nodes.length; i--;)
                    {
                        var node = app.nodes[i];
                        
                        console.log(app.name.blue + (' - '+(node.statename)+' '+node.description).green);
                    }
                }
            });*/
        });
     },
    
    
    
    /**
     */
     
    ':appName history': function(data)
    {
        
        service.request('/garden/apps/'+data.appName+'/checkout/history', function(err, response)
        {
            if(err || response.errors) return unableToFetch();
            
            if(!response.result.length) return console.log('No commits');
            
            
            celery.drawTable(response.result, {
                columns: {
                    label: {
                        width: 20,
                        get: function(checkout)
                        {
                            return data.appName + '@' + checkout.package.version;
                        }
                    },
                    lastUsed: {
                        get: function(checkout)
                        {
                            return new Date(checkout.lastUsed);
                        }
                    }
                }
                
                
            });
        });
    },
    
    /**
     * node-inspector
     */
     
    'debug :appName': function()
    {
        //test
    },
    
    /**
     * good for v1 v2 apis
     */

    'clone :appName :newName': function()
    {
        console.log('cloning');
    },
    
    /**
     */
     
    ':appName rollback': function(data)
    {
        
        service.request('/garden/apps/'+data.appName+'/rollback', function(err, response)
        {
            if(err || response.errors) return unableToFetch();
            
            console.log('Rolling back to previous version');
        });
    },
    
    /**
     * returns the app logs
     */
     
    ':appName logs': function(data)
    {
        console.log("LOGS");
    },
    
    /**
     */
     
    ':appName processes': function(data)
    {
    },
    
    /**
     */
     
    ':appName checkout :repository': function(data)
    {
        celery.emit(data.appName+' checkout', { repository: data.repository });
    },
    
    /**
     */
     
    ':appName checkout': function(data)
    {   
        if(data.repository) data.repository = decodeURIComponent(data.repository);

        simpleLoad({ path: '/garden/apps/'+data.appName+'/checkout', method: 'post', data: { repository: data.repository }}, 'Checking out '+data.appName);
    },
    
    /**
     * Starts interactive console with app - stdin
     */
     
    ':appName console': function(data)
    {
        console.log("TODO");
    },
    
    /**
     */
     
    ':appName logs tail': function(data)
    {
        console.log("TAIL LOGS");
    },
    
    /**
     * starts an app
     */
     
    'start :appName': function(data)
    {
        simpleLoad('/garden/apps/'+data.appName+'/start', 'Starting ' + data.appName);
    },
    
    
    /**
     * stops an app
     */
     
    'stop :appName': function(data)
    {
        simpleLoad('/garden/apps/'+data.appName+'/stop', 'Stopping ' + data.appName);
    },
    
    /** 
     * restart target app
     */
     
    'restart :appName': function(data)
    {
        simpleLoad('/garden/apps/'+data.appName+'/restart', 'Restarting ' + data.appName);
    },
    
    
    
    /**
     * creates a new app
     */
     
    'create :appName': function(data)
    {
        service.request({ path: '/garden/apps/', method: 'post', data: { name: data.appName } }, function(err, result)
        {
            if(result.errors) return console.log('Unable to create app'.red);
            
            console.log('Successfuly created app');
        });
    },
    
    /**
     */
     
    'remove :appName': function(data)
    {
        service.request({ path: '/garden/apps/' + data.appName+'/remove' }, function(err, result)
        {
            if(result.errors) return console.log('Unable to delete'.red);
            
            console.log('Successfuly deleted app');
        })
    },
    
    
    /**
     * restart all
     */
     
    'restart': function()
    {
        console.log("TODO");
    },
    /**
     */
     
    'help': function()
    {
        //celery.openHelp(__dirname + '/help.txt');
    },
    
    /**
     * shovel version
     */
     
    'version': function()
    {
    },
    
    /**
     */
     
    'signup': function()
    {
        celery.auth(function(user, pass)
        {
            console.log(user);
            console.log(pass);
        });
    },
    
    /**
     */
     
    'logout': function()
    {
        console.log("logged out".grey);
        auth();
    }
});


function signup(user, pass)
{
    service.signup(function(err, result)
    {
        if(err || result.errors)
        {
            console.error('Unable to signup'.red);
        }
        else
        {
            loggedIn();
        }
    });
}

function loggedIn()
{
    console.ok('logged in as %s'.green, service.user);
}


console.log('First time logging in? enter in new credentials and click "y"');


function auth()
{
    celery.auth(function(user, pass)
    {
        service.setAuth(user, pass);
        
        
        service.validate(function(err, result)
        {
            if(err) return console.log('Unable to validate credentials'.red); 
            
            if(result.errors)
            {
                celery.confirm('Auth credits are incorrect. Would you like to signup?', function(yes)
                {
                    if(yes) return signup(user, pass);
                    
                    auth();
                });
            }
            else
            {
                loggedIn();
            }
        });
    });
}


auth();




celery.open();

celery.parse(process.argv);